import networkx as nx
from ean_data import *
from typing import Tuple


def cycle_base(ean: 'Ean') -> Dict['EanActivity', List[Tuple['EanActivity', int]]]:
    """
    Class to create a cycle base for a given EAN
    :param ean: the event activity network
    :return: cycle base
    """
    # creating a networkx graph from ean
    EAN = nx.Graph()
    for event in ean.get_events_network():
        EAN.add_node(event.get_event_id(), name=event)
    for activity in ean.get_activities(['drive', 'wait', 'trans', 'sync']):
        left_event = activity.left_event.get_event_id()
        right_event = activity.right_event.get_event_id()
        span = activity.upper_bound - activity.lower_bound
        EAN.add_edge(left_event, right_event, span=span, name=activity)

    T = nx.minimum_spanning_tree(EAN, weight='span')
    C = {}
    for e in EAN.edges():
        if T.has_edge(*e):
            continue
        P = [(EAN[e[0]][e[1]]['name'], 1)]
        if EAN.node[e[0]]['name'] == EAN[e[0]][e[1]]['name'].left_event:
            k = 1
        else:
            k = -1
        prev_node = -1
        for node in nx.shortest_path(T, e[0], e[1]):
            if prev_node != -1:
                if EAN.node[prev_node]['name'] == EAN[prev_node][node]['name'].left_event:
                    P.append((EAN[prev_node][node]['name'], -1 * k))
                else:
                    P.append((EAN[prev_node][node]['name'], 1 * k))
            prev_node = node
        C[EAN[e[0]][e[1]]['name']] = P
    return C





