import os


class ReadConfig:
    """
    Class representing the configuration.
    """

    def __init__(self, config_file_name: str) -> None:
        """
        Initialising an configuration.
        :param config_file_name: name of the main configuration file
        """
        self.parameters = {}
        self.read_config_file(config_file_name)

    def read_config_file(self, config_file_name: str, only_if_exists: bool = False) -> None:
        """
        Method for reading configuration files
        :param config_file_name: name of the configuration file
        :param only_if_exists: whether the file is necessary
        :return: None
        """
        if only_if_exists:
            try:
                open(config_file_name, 'r')
            except IOError:
                return
        config_file = open(config_file_name, 'r')
        parent_dir_name = os.path.dirname(config_file_name)
        for line in config_file:
            line = line.split('#')[0]
            if line:
                words = line.split(";")
                parameter = words[0].strip()
                value = words[1].strip()
                if parameter == "include":
                    self.read_config_file(parent_dir_name + "/" + value.strip('\"'))
                elif parameter == "include_if_exists":
                    self.read_config_file(parent_dir_name + "/" + value.strip('\"'), True)
                else:
                    self.parameters[parameter] = value

    def get_str_param(self, param: str) -> str:
        return str(self.parameters[param].strip('\"'))

    def get_int_param(self, param: str) -> int:
        return int(self.parameters[param])

    def get_float_param(self, param: str) -> float:
        return float(self.parameters[param])

    def get_bool_param(self, param: str) -> bool:
        if self.parameters[param] in ["False", "false"]:
            return False
        elif self.parameters[param] in ["True", "true"]:
            return True
        return bool(self.parameters[param])
