from ptn_data import *
from line_data import *
from od_data import *
import os


def read_ptn(stop_file_name: str, edge_file_name: str, ptn: Ptn, directed: bool, global_min_wait_time: int,
             global_max_wait_time: int, global_min_trans_time: int, global_max_trans_time: int,
             global_min_depot_time: int = 0) -> None:
    stop_file = open(stop_file_name, 'r')
    stop_id_index = 0
    x_coordinate = 3
    y_coordinate = 4
    for line in stop_file:
        line = line.split('#')[0]
        if not line:
            continue
        else:
            words = line.split(";")
            PtnNode(ptn, int(words[stop_id_index]), global_min_wait_time, global_max_wait_time,
                    global_min_trans_time, global_max_trans_time, float(words[x_coordinate]),
                    float(words[y_coordinate]), global_min_depot_time, global_min_depot_time)
    stop_file.close()

    edge_file = open(edge_file_name, 'r')
    edge_id_index = 0
    left_node_id_index = 1
    right_node_id_index = 2
    min_drive_time_index = 4
    max_drive_time_index = 5
    for line in edge_file:
        line = line.split('#')[0]
        if not line:
            continue
        else:
            words = line.split(";")
            left_node = ptn.get_node(int(words[left_node_id_index]))
            right_node = ptn.get_node(int(words[right_node_id_index]))
            PtnEdge(int(words[edge_id_index]), left_node, right_node, int(words[min_drive_time_index]),
                    int(words[max_drive_time_index]), directed)
    edge_file.close()


def read_line_pool(line_pool_file_name: str, line_pool: LinePool, ptn: Ptn, directed: bool,
                   system_frequency: int = 1, read_line_concept: bool = False,
                   restrict_to_frequency_1: bool = True) -> None:
    pool_file = open(line_pool_file_name, 'r')
    line_id_index = 0
    edge_id_index = 2
    frequency_index = 3
    current_id = 0
    new_lines = []
    new_backward_lines = []
    for line in pool_file:
        line = line.split('#')[0]
        if not line:
            continue
        else:
            words = line.split(";")
            line_id = int(words[line_id_index])
            if read_line_concept:
                frequency = int(words[frequency_index])
            else:
                frequency = system_frequency
            if read_line_concept and restrict_to_frequency_1 and frequency > 1:
                raise Exception("Line concepts with frequencies > 1 are not supported!")
            if not read_line_concept or int(words[frequency_index]) >= 1:
                add_line = True
                if line_id != current_id:
                    new_lines = []
                    for repetition in range(1, frequency + 1):
                        new_lines.append(Line(line_id, line_pool, frequency=frequency, repetition=repetition))
                    if not directed:
                        new_backward_lines = []
                        for repetition in range(1, frequency + 1):
                            new_backward_lines.append((Line(-line_id, line_pool, undirected_line_id=line_id,
                                                            frequency=frequency, repetition=repetition)))
                    current_id = line_id
            else:
                add_line = False
            if add_line:
                edge = ptn.get_edge(int(words[edge_id_index]))
                for new_line in new_lines:
                    new_line.add_edge(edge, directed)
                if not directed:
                    for new_line_backwards in new_backward_lines:
                        new_line_backwards.add_edge(edge, directed, False)
    pool_file.close()


def read_od_matrix(od_file_name: str, ptn: Ptn, od: OD, global_n_time_slices: int = 1, global_max_travel_time: int = -1,
                   global_min_turn_around_time: int = -1, period_length: int = 60) -> None:
    od_file = open(od_file_name, 'r')
    origin_id_index = 0
    destination_id_index = 1
    n_passengers_index = 2
    for line in od_file:
        line = line.split('#')[0]
        if not line:
            continue
        else:
            words = line.split(";")
            origin = ptn.get_node(int(words[origin_id_index]))
            destination = ptn.get_node(int(words[destination_id_index]))
            n_passengers = float(words[n_passengers_index])
            n_passengers = n_passengers * period_length / 60
            max_travel_time = global_max_travel_time
            min_turn_around_time = global_min_turn_around_time
            od_pair = ODPair(od, origin, destination, min_turn_around_time, -1, global_n_time_slices, max_travel_time,
                             active=False)
            for t in range(1, od_pair.get_n_time_slices() + 1):
                od_pair.set_n_passengers(t, int(n_passengers / od_pair.get_n_time_slices()))
    od_file.close()
